use actix::{Actor, Addr};
use anyhow::Error;
use redis::{aio::MultiplexedConnection, Client};
use sqlx::PgPool;
use std::net::SocketAddr;

use crate::{cfg::Cfg, live_hub::LiveHub};

#[derive(Clone)]
pub struct Ctx {
    pub cfg: Cfg,
    db: PgPool,
    cache: MultiplexedConnection,
    hub: Addr<LiveHub>,
}

impl Ctx {
    pub async fn new(cfg: Cfg) -> Result<Self, Error> {
        let db = PgPool::connect(&cfg.server.database_url).await?;
        let client = Client::open(cfg.server.cache_url.as_str())?;
        let cache = client.get_multiplexed_async_connection().await?;
        let sync_cache = client.get_connection()?;
        let hub = LiveHub::new(sync_cache).start();

        Ok(Self {
            cfg,
            db,
            cache,
            hub,
        })
    }

    pub fn bind_addr(&self) -> SocketAddr {
        SocketAddr::from(([127, 0, 0, 1], self.cfg.server.port))
    }

    pub fn db(&self) -> &PgPool {
        &self.db
    }

    pub fn cache(&self) -> MultiplexedConnection {
        self.cache.clone()
    }

    pub fn hub(&self) -> Addr<LiveHub> {
        self.hub.clone()
    }
}
