use ::captcha::{gen, Difficulty};
use actix_web::{
    get,
    web::{Data, Json, Query},
};
use redis::AsyncCommands;
use serde::{Deserialize, Serialize};
use sha256::digest;

use crate::{ctx::Ctx, db::models::Board, error::NekrochanError};

#[derive(Deserialize)]
pub struct CaptchaQuery {
    pub board: String,
    pub reply: bool,
}

#[derive(Serialize)]
pub struct CaptchaResponse {
    pub png: String,
    pub id: String,
}

#[get("/captcha")]
pub async fn captcha(
    ctx: Data<Ctx>,
    Query(query): Query<CaptchaQuery>,
) -> Result<Json<CaptchaResponse>, NekrochanError> {
    let board = Board::read(&ctx, query.board.clone())
        .await?
        .ok_or(NekrochanError::BoardNotFound(query.board))?;

    let captcha = match board.config.thread_captcha.as_str() {
        "easy" => gen(Difficulty::Easy),
        "medium" => gen(Difficulty::Medium),
        "hard" => gen(Difficulty::Hard),
        _ => return Err(NekrochanError::NoCaptchaError),
    };

    // >YOU NEED TO MAKE A NEW ERROR TYPE FOR THIS ERROR THAT CAN ONLY HAPPEN ONCE IN THE CODE OR HOWEVER THE TRANS CHILDREN ARE PROTECTED
    let png = captcha.as_base64().ok_or(NekrochanError::NoCaptchaError)?;

    let board = board.id;
    let id = digest(png.as_bytes());

    let key = format!("captcha:{board}:{id}");
    let solution = captcha.chars_as_string();

    ctx.cache().set(&key, solution).await?;
    ctx.cache().expire(&key, 600).await?;

    let res = CaptchaResponse { png, id };

    Ok(Json(res))
}
