use actix_web::{get, web::Data, HttpRequest, HttpResponse};
use askama::Template;

use crate::{
    ctx::Ctx,
    db::models::Board,
    error::NekrochanError,
    filters,
    web::{tcx::TemplateCtx, template_response},
};

#[derive(Template)]
#[template(path = "staff/boards.html")]
struct BoardsTemplate {
    tcx: TemplateCtx,
    boards: Vec<Board>,
}

#[get("/staff/boards")]
pub async fn boards(ctx: Data<Ctx>, req: HttpRequest) -> Result<HttpResponse, NekrochanError> {
    let tcx = TemplateCtx::new(&ctx, &req).await?;

    if !(tcx.perms.owner() || tcx.perms.board_config() || tcx.perms.banners()) {
        return Err(NekrochanError::InsufficientPermissionError);
    }

    let boards = Board::read_all(&ctx).await?;
    let template = BoardsTemplate { tcx, boards };

    template_response(&template)
}
